#ifndef GLPANEL_H
#define GLPANEL_H

#include <vector>
#include <map>
using namespace std;

#include <qgl.h>
#include <QMouseEvent>
//#include <QKeyEvent>
#include <QtOpenGL/QGLWidget>

#include "geommodel.h"
#include "curvecollector.h"

class GLPanel : public QGLWidget
{
 Q_OBJECT // must include this if you use Qt Signals & Slots

 public:
  enum MouseActionType{ UNDEFINED, SELECTION, COLLECTION };

  GLPanel(QWidget *parent = 0);
  ~GLPanel();
  void       setModel(GeomModel* model);
  GeomModel* getModel() { return m; };
  void       resetModelDisplay();
  void       setCurveType( int _type );
  void       setMouseAction( int _action );
  void       setGridSnapData( bool _viewGrid, bool _isSnapOn, double _dx, double _dy );
  bool       getGridSnapInfo( double& _dx, double& _dy );
  void       delSelectedEntities();
  void       intersectTwoSelectedLines();
  void       createRegion();
  void       fitWorldToViewport();
  void       zoomIn();
  void       zoomOut();
  void       panLeft();
  void       panRight();
  void       panDown();
  void       panUp();

public slots:
  // Mouse events slots
  void mousePressEvent(QMouseEvent* event);
  void mouseMoveEvent(QMouseEvent* event);
  void mouseReleaseEvent(QMouseEvent* event);
  // Keyboard event slot
  void keyPressEvent( QKeyEvent* event );
  void keyReleaseEvent( QKeyEvent* event );

 protected:
  GeomModel* m;              // model to be displayed
  GLuint model_dsp_id;       // GL list index for model display
  bool updated_model_dsp;    // if true, model display is updated
  CurveCollector collector;  // Collector of entities on canvas

  int w;                     // width: GL canvas horizontal size
  int h;                     // height: GL canvas vertical size
  GLdouble left;             // left limit of object space window
  GLdouble right;            // right limit of object space window
  GLdouble bottom;           // bottom limit of object space window
  GLdouble top;              // top limit of object space window

  // Mouse point properties
  int curMouseAction;     // current mouse action type
  bool button_pressed;    // if true, mouse button is pressed
  // mouse button that was pressed:
  //(Qt::LeftButton, Qt::RightButton, or Qt::MidButton)
  Qt::MouseButton mouse_button;
  QPoint pt0;             // position at mouse button press event
  QPoint pt1;             // current mouse position
  double pick_tol_fac;    // factor for pick tolerance
  int mouse_move_tol;     // tolerance for mouse move

  // Pressed key properties
  bool shift_key_pressed;
  bool control_key_pressed;

  // Defined colors
  QColor colorBackground;
  QColor colorCollecting;
  QColor colorCurve;
  QColor colorRegion;
  QColor colorVertex;
  QColor colorSelection;
  QColor colorRegionSelection;
  QColor colorGrid;
  QColor colorSdvPoint;
  QColor colorMesh;

  // Canvas predefined slots
  void initializeGL();
  void resizeGL(int width, int height);
  void paintGL();

  // Display functions
  GLuint makeDisplayModel();
  void   drawRegionMeshes();
  GLuint drawSelectionFence();
  GLuint drawCollectedCurve();

  // Functions to manage visualization window limits
  void scaleWorldWindow(double scale_fac);
  void panWorldWindow(double pan_facX, double pan_facY);

  // QPointF is a Qt class that defines a point in floating
  // point coordinates.
  QPointF convertPtCoordsToUniverse(QPoint _pt);

  // Grid data and methods
  bool m_viewGrid;
  bool m_isSnapOn;
  double m_gridX;
  double m_gridY;
  GLuint grid_dsp_id;       // GL list index for grid display
  bool updated_grid_dsp;    // if true, grid display is updated
  GLuint makeDisplayGrid();
  void snapToGrid( QPointF& _pt );
};

#endif // GLPANEL_H
