#include "predicate.h"
#include "point.h"


// Predicates Header Files
extern "C" 
{
#include "predicates.h"
}



//-------------------------------------------------------------------------
SIGN 
Predicate::myorient2d( Point* _p1, Point* _p2, Point* _p3 )
{
  // we test here several different implementations
  // type = 1:my  2:adaptive  3:exact  4:exact slow  5:non robust
  int type = 1;
  double det = 0.0;
  switch( type )
  {
    case 1:
      {
      /*
      Line Equation P(t)=P0+(P1-P0)t
      Px(t)=P0x+(P1x-P0x)t    
      Py(t)=P0y+(P1y-P0y)t   
      orient2d
      x0  y0  1    ->     x0-xt  y0-yt    =  [(x0-xt)(y1-yt)-(y0-yt)(x1-xt)]
      x1  y1  1           x1-xt  y1-yt 
      xt  yt  1
      orient2d = [(x0-xt)(y1-yt)-(y0-yt)(x1-xt)]
      */
      double x0 = _p1->getX();
      double y0 = _p1->getY();
      double x1 = _p2->getX();
      double y1 = _p2->getY();
      double xt = _p3->getX();
      double yt = _p3->getY();
      det = (x0-xt)*(y1-yt)-(y0-yt)*(x1-xt);
//       if (det<0) {
//         //cout << ".est abaixo da reta \n";
//       }
//       else if(det>0) {
//         //cout << ".est acima da reta \n";
//       }
//       else if (det==0) {
//         //cout << ".pertence a reta  \n";
//       }
      break;
      }
    case 2:
      {
        // to start Shewchuk predicates
        exactinit();
      double pa[2] = {_p1->getX(),_p1->getY()};
      double pb[2] = {_p2->getX(),_p2->getY()};
      double pc[2] = {_p3->getX(),_p3->getY()};
      det = orient2d( pa, pb, pc );
      break;
      }
    case 3:
      {
        // to start Shewchuk predicates
        exactinit();
      double pa[2] = {_p1->getX(),_p1->getY()};
      double pb[2] = {_p2->getX(),_p2->getY()};
      double pc[2] = {_p3->getX(),_p3->getY()};
      det = orient2dexact( pa, pb, pc );
      break;
      }
    case 4:
      {
        // to start Shewchuk predicates
        exactinit();
      double pa[2] = {_p1->getX(),_p1->getY()};
      double pb[2] = {_p2->getX(),_p2->getY()};
      double pc[2] = {_p3->getX(),_p3->getY()};
      det = orient2dslow( pa, pb, pc );
      break;
      }
    case 5:
      {
        // to start Shewchuk predicates
        exactinit();
      double pa[2] = {_p1->getX(),_p1->getY()};
      double pb[2] = {_p2->getX(),_p2->getY()};
      double pc[2] = {_p3->getX(),_p3->getY()};
      det = orient2dfast( pa, pb, pc );
      break;
      }
  }
  if( det != 0.0 )
  {
    if( det > 0.0 )
    {
      return POSITIVE;
    } 
    else
    {
      return NEGATIVE;
    }
  } 
  return ZERO;
}
//-------------------------------------------------------------------------


