#include "simplemesh.h"
#include "mesh/meshgenerator.h"

SimpleMesh::SimpleMesh(QWidget *parent, Qt::WFlags flags)
    : QMainWindow(parent, flags)
{
  ui.setupUi(this);

  // create a model object and store a pointer to it in canvas object
  GeomModel* m = new GeomModel();
  ui.canvas->setModel(m);

  // create and setup the dialog responsible for the grid and snap options
  m_gridDialog = new QDialog;
  m_gridDlg.setupUi( m_gridDialog );

  // Set default button checked status
  ui.actionSelect->setChecked(true);
  ui.actionLine->setChecked(false);

  // set default mouse action on canvas
  ui.canvas->setMouseAction( GLPanel::SELECTION );
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
SimpleMesh::~SimpleMesh()
{
  delete m_gridDialog;
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionSelect_triggered()
{
  ui.actionSelect->setChecked(true);
  ui.actionLine->setChecked(false);
  // set corresponding mouse action on canvas
  ui.canvas->setMouseAction( GLPanel::SELECTION );
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionLine_triggered()
{
  ui.actionSelect->setChecked(false);
  ui.actionLine->setChecked(true);
  // set corresponding mouse action on canvas
  ui.canvas->setMouseAction( GLPanel::COLLECTION );
  ui.canvas->setCurveType( Curve::LINE );
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionGrid_triggered()
{
  // verifies if the grid button is turned on or off
  bool viewGrid = ui.actionGrid->isChecked();
  // get data from grid/snap and set in the dialog
  double dx = 0.0;
  double dy = 0.0;
  bool isSnapOn = ui.canvas->getGridSnapInfo( dx, dy );
  QString sdx;
  sdx.setNum( dx );
  m_gridDlg.lineEditXdir->clear();
  m_gridDlg.lineEditXdir->insert( sdx );
  QString sdy;
  sdy.setNum( dy );
  m_gridDlg.lineEditYdir->clear();
  m_gridDlg.lineEditYdir->insert( sdy );
  m_gridDlg.checkBoxSnap->setChecked( viewGrid );
  if( viewGrid )
  {
    // opens dialog and get info from grid and snap
    if( m_gridDialog->exec() == QDialog::Accepted )
    {
      // set info to activate grid and snap
      sdx = m_gridDlg.lineEditXdir->text();
      sdy = m_gridDlg.lineEditYdir->text();
      isSnapOn = m_gridDlg.checkBoxSnap->isChecked();
      ui.canvas->setGridSnapData( viewGrid, isSnapOn, sdx.toDouble(), sdy.toDouble() );
    }
    else
    {
      ui.actionGrid->setChecked( false );
    }
  }
  else
  {
    // set info to deactivate grid and snap
    ui.canvas->setGridSnapData( viewGrid, isSnapOn, dx, dy  );
  }
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionDelete_triggered()
{
  ui.canvas->delSelectedEntities();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionIntersect_triggered()
{
  ui.canvas->intersectTwoSelectedLines();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionCreateRegion_triggered()
{
  ui.canvas->createRegion();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionFit_triggered()
{
  ui.canvas->fitWorldToViewport();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionZoomIn_triggered()
{
  ui.canvas->zoomIn();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionZoomOut_triggered()
{
  ui.canvas->zoomOut();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionPanLeft_triggered()
{
  ui.canvas->panLeft();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionPanRight_triggered()
{
  ui.canvas->panRight();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionPanDown_triggered()
{
  ui.canvas->panDown();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionPanUp_triggered()
{
  ui.canvas->panUp();
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionBoundaryMesh_triggered()
{
  Ui::MeshBoundary dlg; 
  QDialog *dialog = new QDialog;
  dlg.setupUi(dialog);
  int nSdv = 1;
  GeomModel* m = ui.canvas->getModel();
  m->getNumSubdivOfFirstSelectedCurve( &nSdv );
  QString s_nSdv;
  s_nSdv.setNum( nSdv );
  dlg.lineEditNumSubdiv->clear();
  dlg.lineEditNumSubdiv->insert( s_nSdv );
  if( dialog->exec() == QDialog::Accepted )
  {
    s_nSdv = dlg.lineEditNumSubdiv->text();
    nSdv = s_nSdv.toInt();
    m->setNumSubdivOfSelectedCurves( nSdv );
    ui.canvas->resetModelDisplay();
    // returns to the selection mode
    this->on_actionSelect_triggered();
  }
}
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
void 
SimpleMesh::on_actionDomainMesh_triggered()
{
  Ui::MeshDomain dlg; 
  QDialog *dialog = new QDialog;
  dlg.setupUi(dialog);
  if( dialog->exec() == QDialog::Accepted )
  {
    int meshGenType = 0;
    if( dlg.radioButtonBilinear->isChecked ())
    {
      meshGenType = MeshGenerator::TRANSFIN_BILINEAR;
    }
    else // if( dlg.radioButtonTrilinear->isChecked ())
    {
       meshGenType = MeshGenerator::TRANSFIN_TRILINEAR;
    }
    GeomModel* m = ui.canvas->getModel();
    m->generateMeshSelectedRegions(meshGenType);
    ui.canvas->resetModelDisplay();
    // returns to the selection mode
    this->on_actionSelect_triggered();
  }
}
//-------------------------------------------------------------------------
